@@ -140,7 +140,7 @@ def main(argv):  # pylint: disable=W0613
         # Salt thin now is available to use
     else:
         scpstat = subprocess.Popen(['/bin/sh', '-c', 'command -v scp']).wait()
-        if not scpstat == 0:
+        if scpstat != 0:
             sys.exit(EX_SCP_NOT_FOUND)
 
         if not os.path.exists(OPTIONS.saltdir):
@@ -200,6 +200,7 @@ def main(argv):  # pylint: disable=W0613
     salt_argv = [
         sys.executable,
         salt_call_path,
+        '--retcode-passthrough',
         '--local',
         '--metadata',
         '--out', 'json',
@@ -214,8 +215,9 @@ def main(argv):  # pylint: disable=W0613
     # Yes, the flush() is necessary.
     sys.stdout.write(OPTIONS.delimiter + '\n')
     sys.stdout.flush()
-    sys.stderr.write(OPTIONS.delimiter + '\n')
-    sys.stderr.flush()
+    if not OPTIONS.tty:
+        sys.stderr.write(OPTIONS.delimiter + '\n')
+        sys.stderr.flush()
     if OPTIONS.cmd_umask is not None:
         old_umask = os.umask(OPTIONS.cmd_umask)
     if OPTIONS.tty:
