@@ -5,7 +5,7 @@ Connection module for Amazon Security Groups
 .. versionadded:: 2014.7.0
 
 :configuration: This module accepts explicit ec2 credentials but can
-    also utilize IAM roles assigned to the instance trough Instance Profiles.
+    also utilize IAM roles assigned to the instance through Instance Profiles.
     Dynamic credentials are then automatically obtained from AWS API and no
     further configuration is necessary. More Information available at:
 
@@ -51,6 +51,7 @@ import logging
 import re
 from distutils.version import LooseVersion as _LooseVersion  # pylint: disable=import-error,no-name-in-module
 import salt.ext.six as six
+from salt.exceptions import SaltInvocationError
 
 log = logging.getLogger(__name__)
 
@@ -66,6 +67,7 @@ except ImportError:
     HAS_BOTO = False
 
 import salt.utils.odict as odict
+from salt.exceptions import SaltInvocationError
 
 
 def __virtual__():
@@ -80,18 +82,18 @@ def __virtual__():
     # a groupId attribute when a GroupOrCIDR object authorizes an IP range
     # Support for Boto < 2.4.0 can be added if needed
     if not HAS_BOTO:
-        return False
+        return (False, 'The boto_secgroup module could not be loaded: boto libraries not found')
     elif _LooseVersion(boto.__version__) < _LooseVersion(required_boto_version):
-        return False
+        return (False, 'The boto_secgroup module could not be loaded: boto library v2.4.0 not found')
     else:
-        __utils__['boto.assign_funcs'](__name__, 'ec2')
+        __utils__['boto.assign_funcs'](__name__, 'ec2', pack=__salt__)
         return True
 
 
 def exists(name=None, region=None, key=None, keyid=None, profile=None,
-           vpc_id=None, group_id=None):
+           vpc_id=None, vpc_name=None, group_id=None):
     '''
-    Check to see if an security group exists.
+    Check to see if a security group exists.
 
     CLI example::
 
@@ -99,13 +101,27 @@ def exists(name=None, region=None, key=None, keyid=None, profile=None,
     '''
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
 
-    group = _get_group(conn, name, vpc_id, group_id, region)
+    group = _get_group(conn, name=name, vpc_id=vpc_id, vpc_name=vpc_name,
+                       group_id=group_id, region=region, key=key, keyid=keyid,
+                       profile=profile)
     if group:
         return True
     else:
         return False
 
 
+def _check_vpc(vpc_id=None, vpc_name=None, region=None, key=None, keyid=None,
+               profile=None):
+    data = __salt__['boto_vpc.get_id'](name=vpc_name, region=region,
+                                       key=key, keyid=keyid, profile=profile)
+    try:
+        return data.get('id')
+    except TypeError:
+        return None
+    except KeyError:
+        return None
+
+
 def _split_rules(rules):
     '''
     Split rules with combined grants into individual rules.
@@ -131,12 +147,23 @@ def _split_rules(rules):
     return split
 
 
-def _get_group(conn, name=None, vpc_id=None, group_id=None, region=None):  # pylint: disable=W0613
+def _get_group(conn=None, name=None, vpc_id=None, vpc_name=None, group_id=None,
+               region=None, key=None, keyid=None, profile=None):  # pylint: disable=W0613
     '''
-    Get a group object given a name, name and vpc_id or group_id. Return a
-    boto.ec2.securitygroup.SecurityGroup object if the group is found, else
+    Get a group object given a name, name and vpc_id/vpc_name or group_id. Return
+    a boto.ec2.securitygroup.SecurityGroup object if the group is found, else
     return None.
     '''
+    if vpc_name and vpc_id:
+        raise SaltInvocationError('The params \'vpc_id\' and \'vpc_name\' '
+                                  'are mutually exclusive.')
+    if vpc_name:
+        try:
+            vpc_id = _check_vpc(vpc_id=vpc_id, vpc_name=vpc_name, region=region,
+                                key=key, keyid=keyid, profile=profile)
+        except boto.exception.BotoServerError as e:
+            log.debug(e)
+            return None
     if name:
         if vpc_id is None:
             log.debug('getting group for {0}'.format(name))
@@ -150,6 +177,11 @@ def _get_group(conn, name=None, vpc_id=None, group_id=None, region=None):  # pyl
                 # a group in EC2-Classic will have vpc_id set to None
                 if group.vpc_id is None:
                     return group
+            # If there are more security groups, and no vpc_id, we can't know which one to choose.
+            if len(filtered_groups) > 1:
+                raise Exception('Security group belongs to more VPCs, specify the VPC ID!')
+            elif len(filtered_groups) == 1:
+                return filtered_groups[0]
             return None
         elif vpc_id:
             log.debug('getting group for {0} in vpc_id {1}'.format(name, vpc_id))
@@ -211,7 +243,8 @@ def _parse_rules(sg, rules):
     return _rules
 
 
-def get_group_id(name, vpc_id=None, region=None, key=None, keyid=None, profile=None):
+def get_group_id(name, vpc_id=None, vpc_name=None, region=None, key=None,
+                 keyid=None, profile=None):
     '''
     Get a Group ID given a Group Name or Group Name and VPC ID
 
@@ -221,15 +254,16 @@ def get_group_id(name, vpc_id=None, region=None, key=None, keyid=None, profile=N
     '''
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
 
-    group = _get_group(conn, name, vpc_id, region)
+    group = _get_group(conn=conn, name=name, vpc_id=vpc_id, vpc_name=vpc_name,
+                       region=region, key=key, keyid=keyid, profile=profile)
     if group:
         return group.id
     else:
         return False
 
 
-def convert_to_group_ids(groups, vpc_id, region=None, key=None, keyid=None,
-                         profile=None):
+def convert_to_group_ids(groups, vpc_id, vpc_name=None, region=None, key=None,
+                         keyid=None, profile=None):
     '''
     Given a list of security groups and a vpc_id, convert_to_group_ids will
     convert all list items in the given list to security group ids.
@@ -248,7 +282,9 @@ def convert_to_group_ids(groups, vpc_id, region=None, key=None, keyid=None,
         else:
             log.debug('calling boto_secgroup.get_group_id for'
                       ' group name {0}'.format(group))
-            group_id = get_group_id(group, vpc_id, region, key, keyid, profile)
+            group_id = get_group_id(name=group, vpc_id=vpc_id,
+                                    vpc_name=vpc_name, region=region,
+                                    key=key, keyid=keyid, profile=profile)
             log.debug('group name {0} has group id {1}'.format(
                 group, group_id)
             )
@@ -258,7 +294,7 @@ def convert_to_group_ids(groups, vpc_id, region=None, key=None, keyid=None,
 
 
 def get_config(name=None, group_id=None, region=None, key=None, keyid=None,
-               profile=None, vpc_id=None):
+               profile=None, vpc_id=None, vpc_name=None):
     '''
     Get the configuration for a security group.
 
@@ -268,7 +304,9 @@ def get_config(name=None, group_id=None, region=None, key=None, keyid=None,
     '''
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
 
-    sg = _get_group(conn, name, vpc_id, group_id, region)
+    sg = _get_group(conn, name=name, vpc_id=vpc_id, vpc_name=vpc_name,
+                    group_id=group_id, region=region, key=key, keyid=keyid,
+                    profile=profile)
     if sg:
         ret = odict.OrderedDict()
         ret['name'] = sg.name
@@ -277,7 +315,7 @@ def get_config(name=None, group_id=None, region=None, key=None, keyid=None,
         ret['group_id'] = sg.id
         ret['owner_id'] = sg.owner_id
         ret['description'] = sg.description
-        # TODO: add support for tags
+        ret['tags'] = sg.tags
         _rules = _parse_rules(sg, sg.rules)
         _rules_egress = _parse_rules(sg, sg.rules_egress)
         ret['rules'] = _split_rules(_rules)
@@ -287,8 +325,8 @@ def get_config(name=None, group_id=None, region=None, key=None, keyid=None,
         return None
 
 
-def create(name, description, vpc_id=None, region=None, key=None, keyid=None,
-           profile=None):
+def create(name, description, vpc_id=None, vpc_name=None, region=None, key=None,
+           keyid=None, profile=None):
     '''
     Create a security group.
 
@@ -298,6 +336,14 @@ def create(name, description, vpc_id=None, region=None, key=None, keyid=None,
     '''
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
 
+    if not vpc_id and vpc_name:
+        try:
+            vpc_id = _check_vpc(vpc_id=vpc_id, vpc_name=vpc_name, region=region,
+                                key=key, keyid=keyid, profile=profile)
+        except boto.exception.BotoServerError as e:
+            log.debug(e)
+            return False
+
     created = conn.create_security_group(name, description, vpc_id)
     if created:
         log.info('Created security group {0}.'.format(name))
@@ -309,7 +355,7 @@ def create(name, description, vpc_id=None, region=None, key=None, keyid=None,
 
 
 def delete(name=None, group_id=None, region=None, key=None, keyid=None,
-           profile=None, vpc_id=None):
+           profile=None, vpc_id=None, vpc_name=None):
     '''
     Delete a security group.
 
@@ -319,7 +365,9 @@ def delete(name=None, group_id=None, region=None, key=None, keyid=None,
     '''
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
 
-    group = _get_group(conn, name, vpc_id, group_id, region)
+    group = _get_group(conn, name=name, vpc_id=vpc_id, vpc_name=vpc_name,
+                       group_id=group_id, region=region, key=key, keyid=keyid,
+                       profile=profile)
     if group:
         deleted = conn.delete_security_group(group_id=group.id)
         if deleted:
@@ -338,8 +386,8 @@ def delete(name=None, group_id=None, region=None, key=None, keyid=None,
 def authorize(name=None, source_group_name=None,
               source_group_owner_id=None, ip_protocol=None,
               from_port=None, to_port=None, cidr_ip=None, group_id=None,
-              source_group_group_id=None, region=None, key=None,
-              keyid=None, profile=None, vpc_id=None, egress=False):
+              source_group_group_id=None, region=None, key=None, keyid=None,
+              profile=None, vpc_id=None, vpc_name=None, egress=False):
     '''
     Add a new rule to an existing security group.
 
@@ -349,7 +397,9 @@ def authorize(name=None, source_group_name=None,
     '''
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
 
-    group = _get_group(conn, name, vpc_id, group_id, region)
+    group = _get_group(conn, name=name, vpc_id=vpc_id, vpc_name=vpc_name,
+                       group_id=group_id, region=region, key=key, keyid=keyid,
+                       profile=profile)
     if group:
         try:
             added = None
@@ -375,21 +425,21 @@ def authorize(name=None, source_group_name=None,
                 log.error(msg)
                 return False
         except boto.exception.EC2ResponseError as e:
-            log.debug(e)
             msg = ('Failed to add rule to security group {0} with id {1}.'
                    .format(group.name, group.id))
             log.error(msg)
+            log.error(e)
             return False
     else:
-        log.debug('Failed to add rule to security group.')
+        log.error('Failed to add rule to security group.')
         return False
 
 
 def revoke(name=None, source_group_name=None,
            source_group_owner_id=None, ip_protocol=None,
            from_port=None, to_port=None, cidr_ip=None, group_id=None,
-           source_group_group_id=None, region=None, key=None,
-           keyid=None, profile=None, vpc_id=None, egress=False):
+           source_group_group_id=None, region=None, key=None, keyid=None,
+           profile=None, vpc_id=None, vpc_name=None, egress=False):
     '''
     Remove a rule from an existing security group.
 
@@ -399,7 +449,9 @@ def revoke(name=None, source_group_name=None,
     '''
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
 
-    group = _get_group(conn, name, vpc_id, group_id, region)
+    group = _get_group(conn, name=name, vpc_id=vpc_id, vpc_name=vpc_name,
+                       group_id=group_id, region=region, key=key, keyid=keyid,
+                       profile=profile)
     if group:
         try:
             revoked = None
@@ -426,11 +478,181 @@ def revoke(name=None, source_group_name=None,
                 log.error(msg)
                 return False
         except boto.exception.EC2ResponseError as e:
-            log.debug(e)
             msg = ('Failed to remove rule from security group {0} with id {1}.'
                    .format(group.name, group.id))
             log.error(msg)
+            log.error(e)
             return False
     else:
-        log.debug('Failed to remove rule from security group.')
+        log.error('Failed to remove rule from security group.')
         return False
+
+
+def _find_vpcs(vpc_id=None, vpc_name=None, cidr=None, tags=None,
+               region=None, key=None, keyid=None, profile=None):
+    '''
+    Given VPC properties, find and return matching VPC ids.
+    Borrowed from boto_vpc; these could be refactored into a common library
+    '''
+    if all((vpc_id, vpc_name)):
+        raise SaltInvocationError('Only one of vpc_name or vpc_id may be '
+                                  'provided.')
+
+    if not any((vpc_id, vpc_name, tags, cidr)):
+        raise SaltInvocationError('At least one of the following must be '
+                                  'provided: vpc_id, vpc_name, cidr or tags.')
+
+    local_get_conn = __utils__['boto.get_connection_func']('vpc')
+    conn = local_get_conn(region=region, key=key, keyid=keyid, profile=profile)
+    filter_parameters = {'filters': {}}
+
+    if vpc_id:
+        filter_parameters['vpc_ids'] = [vpc_id]
+
+    if cidr:
+        filter_parameters['filters']['cidr'] = cidr
+
+    if vpc_name:
+        filter_parameters['filters']['tag:Name'] = vpc_name
+
+    if tags:
+        for tag_name, tag_value in six.iteritems(tags):
+            filter_parameters['filters']['tag:{0}'.format(tag_name)] = tag_value
+
+    vpcs = conn.get_all_vpcs(**filter_parameters)
+    log.debug('The filters criteria {0} matched the following VPCs:{1}'.format(filter_parameters, vpcs))
+
+    if vpcs:
+        return [vpc.id for vpc in vpcs]
+    else:
+        return []
+
+
+def set_tags(tags,
+             name=None,
+             group_id=None,
+             vpc_name=None,
+             vpc_id=None,
+             region=None,
+             key=None,
+             keyid=None,
+             profile=None):
+    '''
+    sets tags on a security group
+
+    .. versionadded:: 2016.3.0
+
+    tags
+        a dict of key:value pair of tags to set on the security group
+
+    name
+        the name of the security gruop
+
+    group_id
+        the group id of the security group (in lie of a name/vpc combo)
+
+    vpc_name
+        the name of the vpc to search the named group for
+
+    vpc_id
+        the id of the vpc, in lieu of the vpc_name
+
+    region
+        the amazon region
+
+    key
+        amazon key
+
+    keyid
+        amazon keyid
+
+    profile
+        amazon profile
+
+    CLI example:
+
+    .. code-block:: bash
+
+        salt myminion boto_secgroup.set_tags "{'TAG1': 'Value1', 'TAG2': 'Value2'}" security_group_name vpc_id=vpc-13435 profile=my_aws_profile
+    '''
+    conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+    secgrp = _get_group(conn, name=name, vpc_id=vpc_id, vpc_name=vpc_name,
+                        group_id=group_id, region=region, key=key, keyid=keyid,
+                        profile=profile)
+
+    if secgrp:
+        if isinstance(tags, dict):
+            secgrp.add_tags(tags)
+        else:
+            msg = 'Tags must be a dict of tagname:tagvalue'
+            raise SaltInvocationError(msg)
+    else:
+        msg = 'The security group could not be found'
+        raise SaltInvocationError(msg)
+    return True
+
+
+def delete_tags(tags,
+                name=None,
+                group_id=None,
+                vpc_name=None,
+                vpc_id=None,
+                region=None,
+                key=None,
+                keyid=None,
+                profile=None):
+    '''
+    deletes tags from a security group
+
+    .. versionadded:: 2016.3.0
+
+    tags
+        a list of tags to remove
+
+    name
+        the name of the security group
+
+    group_id
+        the group id of the security group (in lie of a name/vpc combo)
+
+    vpc_name
+        the name of the vpc to search the named group for
+
+    vpc_id
+        the id of the vpc, in lieu of the vpc_name
+
+    region
+        the amazon region
+
+    key
+        amazon key
+
+    keyid
+        amazon keyid
+
+    profile
+        amazon profile
+
+    CLI example:
+
+    .. code-block:: bash
+
+        salt myminion boto_secgroup.delete_tags ['TAG_TO_DELETE1','TAG_TO_DELETE2'] security_group_name vpc_id=vpc-13435 profile=my_aws_profile
+    '''
+    conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+    secgrp = _get_group(conn, name=name, vpc_id=vpc_id, vpc_name=vpc_name,
+                        group_id=group_id, region=region, key=key, keyid=keyid,
+                        profile=profile)
+    if secgrp:
+        if isinstance(tags, list):
+            tags_to_remove = {}
+            for tag in tags:
+                tags_to_remove[tag] = None
+            secgrp.remove_tags(tags_to_remove)
+        else:
+            msg = 'Tags must be a list of tagnames to remove from the security group'
+            raise SaltInvocationError(msg)
+    else:
+        msg = 'The security group could not be found'
+        raise SaltInvocationError(msg)
+    return True
