@@ -127,7 +127,7 @@ def upgrade_available(name):
     return latest_version(name) != ''
 
 
-def list_upgrades(refresh=True):
+def list_upgrades(refresh=True, **kwargs):  # pylint: disable=W0613
     '''
     List all available package upgrades on this system
 
@@ -143,7 +143,7 @@ def list_upgrades(refresh=True):
     ret = {}
     for name, data in six.iteritems(get_repo_data().get('repo', {})):
         if version(name):
-            latest = latest_version(name)
+            latest = latest_version(name, refresh=False)
             if latest:
                 ret[name] = latest
     return ret
@@ -228,6 +228,10 @@ def list_pkgs(versions_as_list=False, **kwargs):
             for x in ('removed', 'purge_desired')]):
         return {}
 
+    if kwargs.get('refresh', False):
+        # _get_name_map() needs a refresh_db if cache is not present
+        refresh_db()
+
     ret = {}
     name_map = _get_name_map()
     for pkg_name, val in six.iteritems(_get_reg_software()):
@@ -584,7 +588,7 @@ def install(name=None, refresh=False, pkgs=None, saltenv='base', **kwargs):
             version_num = _get_latest_pkg_version(pkginfo)
 
         # Check if the version is already installed
-        if version_num == old.get(pkg_name) \
+        if version_num in old.get(pkg_name, '').split(',') \
                 or (pkg_name in old and old[pkg_name] == 'Not Found'):
             # Desired version number already installed
             ret[pkg_name] = {'current': version_num}
@@ -870,7 +874,7 @@ def remove(name=None, pkgs=None, version=None, **kwargs):
             ret[target] = {'current': 'not installed'}
             continue
         else:
-            if not version_num == old.get(target) \
+            if version_num not in old.get(target, '').split(',') \
                     and not old.get(target) == "Not Found" \
                     and version_num != 'latest':
                 log.error('{0} {1} not installed'.format(target, version))
