@@ -85,9 +85,9 @@ these states. Here is some example SLS:
     installed if it is not present.
 
 '''
-from __future__ import absolute_import
 
-# Import python libs
+# Import Python libs
+from __future__ import absolute_import
 import sys
 
 # Import salt libs
@@ -270,8 +270,19 @@ def managed(name, ppa=None, **kwargs):
         ret['comment'] = 'Repo management not implemented on this platform'
         return ret
 
+    if 'key_url' in kwargs and ('keyid' in kwargs or 'keyserver' in kwargs):
+        ret['result'] = False
+        ret['comment'] = 'You may not use both "keyid"/"keyserver" and ' \
+                         '"key_url" argument.'
+    if 'repo' in kwargs:
+        ret['result'] = False
+        ret['comment'] = ('\'repo\' is not a supported argument for this '
+                          'state. The \'name\' argument is probably what was '
+                          'intended.')
+        return ret
+
     repo = name
-    if __grains__['os'] == 'Ubuntu':
+    if __grains__['os'] in ('Ubuntu', 'Mint'):
         if ppa is not None:
             # overload the name/repo value for PPAs cleanly
             # this allows us to have one code-path for PPAs
@@ -300,7 +311,7 @@ def managed(name, ppa=None, **kwargs):
     if kwargs.pop('enabled', None):
         kwargs['disabled'] = False
         salt.utils.warn_until(
-            'Boron',
+            'Carbon',
             'The `enabled` argument has been deprecated in favor of '
             '`disabled`.'
         )
@@ -417,7 +428,7 @@ def managed(name, ppa=None, **kwargs):
     except Exception as exc:
         ret['result'] = False
         ret['comment'] = \
-            'Failed to confirm config of repo \'{0}\': {1}'.format(name, exc)
+            'Failed to confirm config of repo {0!r}: {1}'.format(name, exc)
 
     # Clear cache of available packages, if present, since changes to the
     # repositories may change the packages that are available.
@@ -481,7 +492,7 @@ def absent(name, **kwargs):
            'changes': {},
            'result': None,
            'comment': ''}
-    repo = {}
+
     if 'ppa' in kwargs and __grains__['os'] in ('Ubuntu', 'Mint'):
         name = kwargs.pop('ppa')
         if not name.startswith('ppa:'):
@@ -546,11 +557,4 @@ def absent(name, **kwargs):
         ret['result'] = False
         ret['comment'] = 'Failed to remove repo {0}'.format(name)
 
-    # Clear cache of available packages, if present, since changes to the
-    # repositories may change the packages that are available.
-    if ret['changes']:
-        sys.modules[
-            __salt__['test.ping'].__module__
-        ].__context__.pop('pkg._avail', None)
-
     return ret
