@@ -15,8 +15,8 @@ file or for the minion via config or pillar)
         kwargs:
           test: True
 
-This will schedule the command: state.sls httpd test=True every 3600 seconds
-(every hour)
+This will schedule the command: ``state.sls httpd test=True`` every 3600 seconds
+(every hour).
 
 .. code-block:: yaml
 
@@ -30,8 +30,8 @@ This will schedule the command: state.sls httpd test=True every 3600 seconds
           test: True
         splay: 15
 
-This will schedule the command: state.sls httpd test=True every 3600 seconds
-(every hour) splaying the time between 0 and 15 seconds
+This will schedule the command: ``state.sls httpd test=True`` every 3600 seconds
+(every hour) splaying the time between 0 and 15 seconds.
 
 .. code-block:: yaml
 
@@ -47,13 +47,14 @@ This will schedule the command: state.sls httpd test=True every 3600 seconds
           start: 10
           end: 15
 
-This will schedule the command: state.sls httpd test=True every 3600 seconds
-(every hour) splaying the time between 10 and 15 seconds
+This will schedule the command: ``state.sls httpd test=True`` every 3600 seconds
+(every hour) splaying the time between 10 and 15 seconds.
 
 .. versionadded:: 2014.7.0
 
 Frequency of jobs can also be specified using date strings supported by
-the python dateutil library.
+the Python ``dateutil`` library. This requires the Python ``dateutil`` library
+to be installed.
 
 .. code-block:: yaml
 
@@ -66,7 +67,7 @@ the python dateutil library.
           test: True
         when: 5:00pm
 
-This will schedule the command: state.sls httpd test=True at 5:00pm minion
+This will schedule the command: ``state.sls httpd test=True`` at 5:00 PM minion
 localtime.
 
 .. code-block:: yaml
@@ -79,15 +80,69 @@ localtime.
         kwargs:
           test: True
         when:
-            - Monday 5:00pm
-            - Tuesday 3:00pm
-            - Wednesday 5:00pm
-            - Thursday 3:00pm
-            - Friday 5:00pm
+          - Monday 5:00pm
+          - Tuesday 3:00pm
+          - Wednesday 5:00pm
+          - Thursday 3:00pm
+          - Friday 5:00pm
 
-This will schedule a job to run once on the specified date. The default date
-format is ISO 8601 but can be overridden by also specifying the ``once_fmt``
-option.
+This will schedule the command: ``state.sls httpd test=True`` at 5:00 PM on
+Monday, Wednesday and Friday, and 3:00 PM on Tuesday and Thursday.
+
+.. code-block:: yaml
+
+    schedule:
+      job1:
+        function: state.sls
+        seconds: 3600
+        args:
+          - httpd
+        kwargs:
+          test: True
+        range:
+          start: 8:00am
+          end: 5:00pm
+
+This will schedule the command: ``state.sls httpd test=True`` every 3600 seconds
+(every hour) between the hours of 8:00 AM and 5:00 PM. The range parameter must
+be a dictionary with the date strings using the ``dateutil`` format.
+
+.. code-block:: yaml
+
+    schedule:
+      job1:
+        function: state.sls
+        seconds: 3600
+        args:
+          - httpd
+        kwargs:
+          test: True
+        range:
+          invert: True
+          start: 8:00am
+          end: 5:00pm
+
+Using the invert option for range, this will schedule the command
+``state.sls httpd test=True`` every 3600 seconds (every hour) until the current
+time is between the hours of 8:00 AM and 5:00 PM. The range parameter must be
+a dictionary with the date strings using the ``dateutil`` format.
+
+.. code-block:: yaml
+
+    schedule:
+      job1:
+        function: pkg.install
+        kwargs:
+          pkgs: [{'bar': '>1.2.3'}]
+          refresh: true
+        once: '2016-01-07T14:30:00'
+
+This will schedule the function ``pkg.install`` to be executed once at the
+specified time. The schedule entry ``job1`` will not be removed after the job
+completes, therefore use ``schedule.delete`` to manually remove it afterwards.
+
+The default date format is ISO 8601 but can be overridden by also specifying the
+``once_fmt`` option, like this:
 
 .. code-block:: yaml
 
@@ -97,67 +152,21 @@ option.
         once: 2015-04-22T20:21:00
         once_fmt: '%Y-%m-%dT%H:%M:%S'
 
-This will schedule the command: state.sls httpd test=True at 5pm on Monday,
-Wednesday and Friday, and 3pm on Tuesday and Thursday.
-
-.. code-block:: yaml
-
-    schedule:
-      job1:
-        function: state.sls
-        seconds: 3600
-        args:
-          - httpd
-        kwargs:
-          test: True
-        range:
-            start: 8:00am
-            end: 5:00pm
-
-This will schedule the command: state.sls httpd test=True every 3600 seconds
-(every hour) between the hours of 8am and 5pm.  The range parameter must be a
-dictionary with the date strings using the dateutil format.
-
 .. versionadded:: 2014.7.0
 
-.. code-block:: yaml
+The scheduler also supports ensuring that there are no more than N copies of
+a particular routine running.  Use this for jobs that may be long-running
+and could step on each other or pile up in case of infrastructure outage.
 
-    schedule:
-      job1:
-        function: state.sls
-        seconds: 3600
-        args:
-          - httpd
-        kwargs:
-          test: True
-        range:
-            invert: True
-            start: 8:00am
-            end: 5:00pm
-
-Using the invert option for range, this will schedule the command: state.sls
-httpd test=True every 3600 seconds (every hour) until the current time is
-between the hours of 8am and 5pm.  The range parameter must be a dictionary
-with the date strings using the dateutil format.
-
-By default any job scheduled based on the startup time of the minion will run
-the scheduled job when the minion starts up.  Sometimes this is not the desired
-situation.  Using the 'run_on_start' parameter set to False will cause the
-scheduler to skip this first run and wait until the next scheduled run.
-
-.. versionadded:: 2015.5.0
+The default for ``maxrunning`` is 1.
 
 .. code-block:: yaml
 
     schedule:
-      job1:
-        function: state.sls
-        seconds: 3600
-        run_on_start: False
-        args:
-          - httpd
-        kwargs:
-          test: True
+      long_running_job:
+        function: big_file_transfer
+        jid_include: True
+        maxrunning: 1
 
 .. versionadded:: 2014.7.0
 
@@ -173,9 +182,60 @@ scheduler to skip this first run and wait until the next scheduled run.
           test: True
 
 The scheduler also supports scheduling jobs using a cron like format.
-This requires the python-croniter library.
+This requires the Python ``croniter`` library.
 
-    ... versionadded:: 2015.8.0
+.. versionadded:: 2015.5.0
+
+By default, data about jobs runs from the Salt scheduler is returned to the
+master. Setting the ``return_job`` parameter to False will prevent the data
+from being sent back to the Salt master.
+
+.. code-block:: yaml
+
+    schedule:
+      job1:
+        function: scheduled_job_function
+        return_job: False
+
+.. versionadded:: 2015.5.0
+
+It can be useful to include specific data to differentiate a job from other
+jobs. Using the metadata parameter special values can be associated with
+a scheduled job. These values are not used in the execution of the job,
+but can be used to search for specific jobs later if combined with the
+``return_job`` parameter. The metadata parameter must be specified as a
+dictionary, othewise it will be ignored.
+
+.. code-block:: yaml
+
+    schedule:
+      job1:
+        function: scheduled_job_function
+        metadata:
+          foo: bar
+
+.. versionadded:: 2015.5.0
+
+By default any job scheduled based on the startup time of the minion will run
+the scheduled job when the minion starts up. Sometimes this is not the desired
+situation. Using the ``run_on_start`` parameter set to ``False`` will cause the
+scheduler to skip this first run and wait until the next scheduled run:
+
+.. code-block:: yaml
+
+    schedule:
+      job1:
+        function: state.sls
+        seconds: 3600
+        run_on_start: False
+        args:
+          - httpd
+        kwargs:
+          test: True
+
+.. versionadded:: 2015.8.0
+
+.. code-block:: yaml
 
     schedule:
       job1:
@@ -188,12 +248,14 @@ This requires the python-croniter library.
           test: True
 
 Using the until argument, the Salt scheduler allows you to specify
-an end time for a scheduled job.  If this argument is specified, jobs
-will not run once the specified time has passed.  Time should be specified
-in a format support by the dateutil library.
-This requires the python-dateutil library.
+an end time for a scheduled job. If this argument is specified, jobs
+will not run once the specified time has passed. Time should be specified
+in a format supported by the ``dateutil`` library.
+This requires the Python ``dateutil`` library to be installed.
 
-    ... versionadded:: 2015.8.0
+.. versionadded:: 2015.8.0
+
+.. code-block:: yaml
 
     schedule:
       job1:
@@ -207,65 +269,28 @@ This requires the python-dateutil library.
 
 Using the after argument, the Salt scheduler allows you to specify
 an start time for a scheduled job.  If this argument is specified, jobs
-will not run until the specified time has passed.  Time should be specified
-in a format support by the dateutil library.
-This requires the python-dateutil library.
-
-The scheduler also supports ensuring that there are no more than N copies of
-a particular routine running.  Use this for jobs that may be long-running
-and could step on each other or pile up in case of infrastructure outage.
-
-The default for maxrunning is 1.
-
-.. code-block:: yaml
-
-    schedule:
-      long_running_job:
-          function: big_file_transfer
-          jid_include: True
-          maxrunning: 1
-
-By default, data about jobs runs from the Salt scheduler is returned to the
-master.  Setting the ``return_job`` parameter to False will prevent the data
-from being sent back to the Salt master.
-
-.. versionadded:: 2015.5.0
-
-    schedule:
-      job1:
-          function: scheduled_job_function
-          return_job: False
-
-It can be useful to include specific data to differentiate a job from other
-jobs.  Using the metadata parameter special values can be associated with
-a scheduled job.  These values are not used in the execution of the job,
-but can be used to search for specific jobs later if combined with the
-return_job parameter.  The metadata parameter must be specified as a
-dictionary, othewise it will be ignored.
-
-.. versionadded:: 2015.5.0
-
-    schedule:
-      job1:
-          function: scheduled_job_function
-          metadata:
-            foo: bar
+will not run until the specified time has passed. Time should be specified
+in a format supported by the ``dateutil`` library.
+This requires the Python ``dateutil`` library to be installed.
 
 '''
 
 # Import python libs
-from __future__ import absolute_import
+from __future__ import absolute_import, with_statement
 import os
+import sys
 import time
+import signal
 import datetime
 import itertools
-import multiprocessing
 import threading
 import logging
 import errno
 import random
+import yaml
 
 # Import Salt libs
+import salt.config
 import salt.utils
 import salt.utils.jid
 import salt.utils.process
@@ -274,11 +299,13 @@ import salt.loader
 import salt.minion
 import salt.payload
 import salt.syspaths
+import salt.exceptions
+import salt.log.setup as log_setup
+import salt.defaults.exitcodes
 from salt.utils.odict import OrderedDict
-from salt.utils.process import os_is_running
+from salt.utils.process import os_is_running, default_signals, SignalHandlingMultiprocessingProcess
 
 # Import 3rd-party libs
-import yaml
 import salt.ext.six as six
 
 # pylint: disable=import-error
@@ -333,16 +360,28 @@ class Schedule(object):
         '''
         Persist the modified schedule into <<configdir>>/minion.d/_schedule.conf
         '''
-        schedule_conf = os.path.join(
-                salt.syspaths.CONFIG_DIR,
-                'minion.d',
-                '_schedule.conf')
+        config_dir = self.opts.get('conf_dir', None)
+        if config_dir is None and 'conf_file' in self.opts:
+            config_dir = os.path.dirname(self.opts['conf_file'])
+        if config_dir is None:
+            config_dir = salt.syspaths.CONFIG_DIR
+
+        minion_d_dir = os.path.join(
+            config_dir,
+            os.path.dirname(self.opts.get('default_include',
+                                          salt.config.DEFAULT_MINION_OPTS['default_include'])))
+
+        if not os.path.isdir(minion_d_dir):
+            os.makedirs(minion_d_dir)
+
+        schedule_conf = os.path.join(minion_d_dir, '_schedule.conf')
         log.debug('Persisting schedule')
         try:
             with salt.utils.fopen(schedule_conf, 'wb+') as fp_:
                 fp_.write(yaml.dump({'schedule': self.opts['schedule']}))
         except (IOError, OSError):
-            log.error('Failed to persist the updated schedule')
+            log.error('Failed to persist the updated schedule',
+                      exc_info_on_loglevel=logging.DEBUG)
 
     def delete_job(self, name, persist=True, where=None):
         '''
@@ -387,6 +426,12 @@ class Schedule(object):
         if not len(data) == 1:
             raise ValueError('You can only schedule one new job at a time.')
 
+        # if enabled is not included in the job,
+        # assume job is enabled.
+        for job in data.keys():
+            if 'enabled' not in data[job]:
+                data[job]['enabled'] = True
+
         new_job = next(six.iterkeys(data))
 
         if new_job in self.opts['schedule']:
@@ -492,13 +537,22 @@ class Schedule(object):
         log.info(
             'Running Job: {0}.'.format(name)
         )
-        if self.opts.get('multiprocessing', True):
-            thread_cls = multiprocessing.Process
+
+        multiprocessing_enabled = self.opts.get('multiprocessing', True)
+        if multiprocessing_enabled:
+            thread_cls = SignalHandlingMultiprocessingProcess
         else:
             thread_cls = threading.Thread
-        proc = thread_cls(target=self.handle_func, args=(func, data))
-        proc.start()
-        if self.opts.get('multiprocessing', True):
+
+        proc = thread_cls(target=self.handle_func, args=(multiprocessing_enabled, func, data))
+        if multiprocessing_enabled:
+            with default_signals(signal.SIGINT, signal.SIGTERM):
+                # Reset current signals before starting the process in
+                # order not to inherit the current signal handlers
+                proc.start()
+        else:
+            proc.start()
+        if multiprocessing_enabled:
             proc.join()
 
     def enable_schedule(self):
@@ -570,7 +624,7 @@ class Schedule(object):
         evt.fire_event({'complete': True},
                        tag='/salt/minion/minion_schedule_saved')
 
-    def handle_func(self, func, data):
+    def handle_func(self, multiprocessing_enabled, func, data):
         '''
         Execute this method in a multiprocess or thread
         '''
@@ -582,6 +636,7 @@ class Schedule(object):
             self.returners = salt.loader.returners(self.opts, self.functions)
         ret = {'id': self.opts.get('id', 'master'),
                'fun': func,
+               'fun_args': [],
                'schedule': data['name'],
                'jid': salt.utils.jid.gen_jid()}
 
@@ -595,7 +650,7 @@ class Schedule(object):
                 log.warning('schedule: The metadata parameter must be '
                             'specified as a dictionary.  Ignoring.')
 
-        salt.utils.appendproctitle(ret['jid'])
+        salt.utils.appendproctitle('{0} {1}'.format(self.__class__.__name__, ret['jid']))
 
         proc_fn = os.path.join(
             salt.minion.get_proc_dir(self.opts['cachedir']),
@@ -642,9 +697,15 @@ class Schedule(object):
                         except OSError:
                             log.info('Unable to remove file: {0}.'.format(fn_))
 
-        try:
-            salt.utils.daemonize_if(self.opts)
+        if multiprocessing_enabled and not salt.utils.is_windows():
+            # Reconfigure multiprocessing logging after daemonizing
+            log_setup.setup_multiprocessing_logging()
 
+        # Don't *BEFORE* to go into try to don't let it triple execute the finally section.
+        salt.utils.daemonize_if(self.opts)
+
+        # TODO: Make it readable! Splt to funcs, remove nested try-except-finally sections.
+        try:
             ret['pid'] = os.getpid()
 
             if 'jid_include' not in data or data['jid_include']:
@@ -657,10 +718,12 @@ class Schedule(object):
             args = tuple()
             if 'args' in data:
                 args = data['args']
+                ret['fun_args'].extend(data['args'])
 
             kwargs = {}
             if 'kwargs' in data:
                 kwargs = data['kwargs']
+                ret['fun_args'].append(data['kwargs'])
 
             if func not in self.functions:
                 ret['return'] = self.functions.missing_fun_string(func)
@@ -679,8 +742,10 @@ class Schedule(object):
 
             data_returner = data.get('returner', None)
             if data_returner or self.schedule_returner:
-                if 'returner_config' in data:
-                    ret['ret_config'] = data['returner_config']
+                if 'return_config' in data:
+                    ret['ret_config'] = data['return_config']
+                if 'return_kwargs' in data:
+                    ret['ret_kwargs'] = data['return_kwargs']
                 rets = []
                 for returner in [data_returner, self.schedule_returner]:
                     if isinstance(returner, str):
@@ -700,7 +765,10 @@ class Schedule(object):
                             )
                         )
 
-            ret['retcode'] = self.functions.pack['__context__']['retcode']
+            # runners do not provide retcode
+            if 'retcode' in self.functions.pack['__context__']:
+                ret['retcode'] = self.functions.pack['__context__']['retcode']
+
             ret['success'] = True
         except Exception:
             log.exception("Unhandled exception running {0}".format(ret['fun']))
@@ -722,11 +790,11 @@ class Schedule(object):
                         # Send back to master so the job is included in the job list
                         mret = ret.copy()
                         mret['jid'] = 'req'
-                        channel = salt.transport.Channel.factory(self.opts, usage='salt_schedule')
+                        event = salt.utils.event.get_event('minion', opts=self.opts, listen=False)
                         load = {'cmd': '_return', 'id': self.opts['id']}
                         for key, value in six.iteritems(mret):
                             load[key] = value
-                        channel.send(load)
+                        event.fire_event(load, '__schedule_return')
 
                 log.debug('schedule.handle_func: Removing {0}'.format(proc_fn))
                 os.unlink(proc_fn)
@@ -740,6 +808,10 @@ class Schedule(object):
                     # Otherwise, failing to delete this file is not something
                     # we can cleanly handle.
                     raise
+            finally:
+                if multiprocessing_enabled:
+                    # Let's make sure we exit the process!
+                    sys.exit(salt.defaults.exitcodes.EX_GENERIC)
 
     def eval(self):
         '''
@@ -783,7 +855,7 @@ class Schedule(object):
 
             if 'until' in data:
                 if not _WHEN_SUPPORTED:
-                    log.error('Missing python-dateutil.'
+                    log.error('Missing python-dateutil. '
                               'Ignoring until.')
                 else:
                     until__ = dateutil_parser.parse(data['until'])
@@ -796,7 +868,7 @@ class Schedule(object):
 
             if 'after' in data:
                 if not _WHEN_SUPPORTED:
-                    log.error('Missing python-dateutil.'
+                    log.error('Missing python-dateutil. '
                               'Ignoring after.')
                 else:
                     after__ = dateutil_parser.parse(data['after'])
@@ -818,7 +890,7 @@ class Schedule(object):
 
             if any(i <= schedule_keys for i in invalid_sched_combos):
                 log.error('Unable to use "{0}" options together. Ignoring.'
-                        .format('", "'.join(scheduling_elements)))
+                          .format('", "'.join(scheduling_elements)))
                 continue
 
             invalid_time_combos = []
@@ -829,8 +901,8 @@ class Schedule(object):
 
             if any(set(x) <= schedule_keys for x in invalid_time_combos):
                 log.error('Unable to use "{0}" with "{1}" options. Ignoring'
-                        .format('", "'.join(time_elements),
-                            '", "'.join(scheduling_elements)))
+                          .format('", "'.join(time_elements),
+                                  '", "'.join(scheduling_elements)))
                 continue
 
             if True in [True for item in time_elements if item in data]:
@@ -857,18 +929,18 @@ class Schedule(object):
 
             elif 'when' in data:
                 if not _WHEN_SUPPORTED:
-                    log.error('Missing python-dateutil.'
-                              'Ignoring job {0}'.format(job))
+                    log.error('Missing python-dateutil. '
+                              'Ignoring job {0}.'.format(job))
                     continue
 
                 if isinstance(data['when'], list):
                     _when = []
                     for i in data['when']:
-                        if ('whens' in self.opts['pillar'] and
+                        if ('pillar' in self.opts and 'whens' in self.opts['pillar'] and
                                 i in self.opts['pillar']['whens']):
                             if not isinstance(self.opts['pillar']['whens'],
                                               dict):
-                                log.error('Pillar item "whens" must be dict.'
+                                log.error('Pillar item "whens" must be dict. '
                                           'Ignoring')
                                 continue
                             __when = self.opts['pillar']['whens'][i]
@@ -894,7 +966,7 @@ class Schedule(object):
                             try:
                                 when__ = dateutil_parser.parse(i)
                             except ValueError:
-                                log.error('Invalid date string {0}.'
+                                log.error('Invalid date string {0}. '
                                           'Ignoring job {1}.'.format(i, job))
                                 continue
                         when = int(time.mktime(when__.timetuple()))
@@ -913,8 +985,8 @@ class Schedule(object):
                             data['_when'] = when
                         seconds = when - now
 
-                        # scheduled time is in the past
-                        if seconds < 0:
+                        # scheduled time is in the past and the run was not triggered before
+                        if seconds < 0 and not data.get('_when_run', False):
                             continue
 
                         if '_when_run' not in data:
@@ -933,7 +1005,7 @@ class Schedule(object):
                         continue
 
                 else:
-                    if ('whens' in self.opts['pillar'] and
+                    if ('pillar' in self.opts and 'whens' in self.opts['pillar'] and
                             data['when'] in self.opts['pillar']['whens']):
                         if not isinstance(self.opts['pillar']['whens'], dict):
                             log.error('Pillar item "whens" must be dict.'
@@ -966,8 +1038,8 @@ class Schedule(object):
                     now = int(time.time())
                     seconds = when - now
 
-                    # scheduled time is in the past
-                    if seconds < 0:
+                    # scheduled time is in the past and the run was not triggered before
+                    if seconds < 0 and not data.get('_when_run', False):
                         continue
 
                     if '_when_run' not in data:
@@ -1023,26 +1095,17 @@ class Schedule(object):
                         else:
                             data['_seconds'] = 0
 
-            if job in self.intervals:
-                if 'when' in data:
-                    if seconds == 0:
-                        if data['_when_run']:
-                            data['_when_run'] = False
-                            run = True
-                elif 'cron' in data:
-                    if seconds == 1:
-                        run = True
-                else:
-                    if now - self.intervals[job] >= seconds:
-                        run = True
+            if 'when' in data:
+                # scheduled time is now or in the past, and the run was triggered before
+                if seconds <= 0 and data['_when_run']:
+                    data['_when_run'] = False
+                    run = True
+            elif 'cron' in data:
+                if seconds == 1:
+                    run = True
             else:
-                if 'when' in data:
-                    if seconds == 0:
-                        if data['_when_run']:
-                            data['_when_run'] = False
-                            run = True
-                elif 'cron' in data:
-                    if seconds == 1:
+                if job in self.intervals:
+                    if now - self.intervals[job] >= seconds:
                         run = True
                 else:
                     # If run_on_start is True, the job will run when the Salt
@@ -1132,6 +1195,8 @@ class Schedule(object):
                              'job {0}, defaulting to 1.'.format(job))
                     data['maxrunning'] = 1
 
+            multiprocessing_enabled = self.opts.get('multiprocessing', True)
+
             if salt.utils.is_windows():
                 # Temporarily stash our function references.
                 # You can't pickle function references, and pickling is
@@ -1141,13 +1206,21 @@ class Schedule(object):
                 returners = self.returners
                 self.returners = {}
             try:
-                if self.opts.get('multiprocessing', True):
-                    thread_cls = multiprocessing.Process
+                if multiprocessing_enabled:
+                    thread_cls = SignalHandlingMultiprocessingProcess
                 else:
                     thread_cls = threading.Thread
-                proc = thread_cls(target=self.handle_func, args=(func, data))
-                proc.start()
-                if self.opts.get('multiprocessing', True):
+                proc = thread_cls(target=self.handle_func, args=(multiprocessing_enabled, func, data))
+
+                if multiprocessing_enabled:
+                    with default_signals(signal.SIGINT, signal.SIGTERM):
+                        # Reset current signals before starting the process in
+                        # order not to inherit the current signal handlers
+                        proc.start()
+                else:
+                    proc.start()
+
+                if multiprocessing_enabled:
                     proc.join()
             finally:
                 self.intervals[job] = now
